#!/bin/bash
#-*-coding:utf8-*-

from common.fromAll_snappy  import *
import xml.etree.ElementTree as ET

class mainWindowSnappy:

    def delete_event(self, widget, event):    
        gtk.main_quit()
        if self.caseDir == home + '/.OpenFOAM/Baram_snappy_temporary':
            os.system('rm -rf ' + self.caseDir)
        elif self.caseDir[:-1] == home + '/.OpenFOAM/Baram_snappy_temporary':
            os.system('rm -rf ' + self.caseDir)

    def destroy(self, widget):
        gtk.main_quit()
        
    def __init__(self, caseDir, installpath):
    
        self.caseDir = caseDir
        self.installpath = installpath
        self.home = home

        self.STY = mainWindowStyleClass(self)
        self.rgb, self.ver, self.pixbuf_main, self.pixbuf_first, self.pixbuf_second = \
            self.STY.rgbVerPixbuf()       
        
        self.VTK_All = VTKStuff(self.caseDir)   
        
        desiredUpdateRate = 1024 
        
        self.gvtk = GtkGLExtVTKRenderWindowInteractor()
        self.gvtk.SetDesiredUpdateRate(desiredUpdateRate)
        self.gvtk.AddObserver("LeftButtonPressEvent", self.leftButtonPressEvent)

        self.cellPicker=vtk.vtkCellPicker()
        self.cellPicker.SetTolerance(0.005)

        self.displayMode = 'Surface'
        self.culling = False
        self.backgroundColor = 'paraview'        
        self.nowPanel = 'Geometry'
        
        self.ver = 'Baram-snappy-v6.4'             
        self.solver = 'snappyHexMesh'
        
        self.stlnames = []
        self.selectedPatch = None
        self.selectedBox = None
        
        self.boxActorDict = {}
        self.Actors = []
                                
    def mainWindowGUI(self):

        self.window, self.Widget_OriMarker = self.STY.mainStyle()
        self.window.set_title(self.ver + ', ' + self.caseDir)
        self.window.connect("delete_event", self.delete_event)        
        self.mainbox = gtk.VBox(False, 0)
        self.window.add(self.mainbox)
        
        self.menubox = gtk.HBox(False, 0)
        self.mainbox.pack_start(self.menubox, False, False, 0)
        self.mb = gtk.MenuBar()
        self.menubox.pack_start(self.mb, True, True, 0)

        toolframe = gtk.Frame()
        toolframe.set_shadow_type(gtk.SHADOW_IN)
        self.mainbox.pack_start(toolframe, False, False, 0)
        self.toolbarbox = gtk.HBox(False, 0)
        toolframe.add(self.toolbarbox)
        
        self.hpan = gtk.HPaned()
        self.mainbox.pack_start(self.hpan, True, True, 0)
        self.freevbox = gtk.VBox(False, 0)
        self.hpan.add(self.freevbox)

        tailbox = gtk.HBox(False, 5)
        self.mainbox.pack_start(tailbox, False, False, 0)
        label = self.ver + ' : OpenFOAM v6 based snappyHexMesh Graphic UI of NEXTFOAM'
        tailbox.pack_end(gtk.Label(label), False, False, 5)

        self.renderbox = gtk.VBox()
        self.gvtk.set_size_request(300, 200)

        toolEvent = gtk.EventBox()
        self.viewToolbox = gtk.HBox(False, 0)
        toolEvent.add(self.viewToolbox)
        self.renderbox.pack_start(toolEvent, False, False, 0)
        
        self.makeToolbar_view()
        
        self.disbox, self.discombo = self.STY.displayModeSnappy() # display mode
        self.viewToolbox.pack_start(self.disbox, False, False, 5)
        self.discombo.connect('changed', self.changeDisplayMode)

        bgbox, self.bgCombo = self.STY.backgroundColor() # background color                  
        self.viewToolbox.pack_start(bgbox, False, False, 5)
        self.bgCombo.connect('changed', self.chbg)

        backgroundRenderer = self.STY.backgroundRenderer()# logo

        self.renderbox.pack_start(self.gvtk, True, True, 0)
        self.renderer = vtk.vtkRenderer()
        self.gvtk.GetRenderWindow().AddRenderer(self.renderer)
        
        self.renderer.GradientBackgroundOn()
        self.renderer.SetBackground(1, 1, 1)
        self.renderer.SetBackground2(self.rgb[0], self.rgb[1], self.rgb[2])

        self.renderer.SetLayer(0)
        self.gvtk.GetRenderWindow().SetNumberOfLayers(2)
        self.gvtk.GetRenderWindow().AddRenderer(backgroundRenderer)

        self.Widget_OriMarker.SetInteractor(self.gvtk)
        self.Widget_OriMarker.EnabledOn()
        self.Widget_OriMarker.InteractiveOff()
        
        self.shellbox = gtk.VBox(False, 0)

        self.notebook = gtk.Notebook()
        self.notebook.set_tab_pos(gtk.POS_TOP)
        self.hpan.add(self.notebook)
        
        self.notebook.append_page(self.renderbox, gtk.Label('Graphic window'))
        self.notebook.append_page(self.shellbox, gtk.Label('Shell window'))

        self.makeToolbar()       
        self.createMenu()
        
        self.setfolder()       
        self.loadXML()
        
        self.cadPart()             
        self.freevbox.pack_start(self.cadbox, True, True, 0)        
        
        self.window.show_all()                   
        
        self.makeTerminal()
        
        self.notebook.set_current_page(1)
    #--------------------------------------------------------------------------------------
    def createMenu(self):
        
        items, menus, menuitems = self.STY.menuStyleSnappy()        
        for i in range(len(items)):
            for j in range(len(items[i])):
                mi = gtk.MenuItem(items[i][j])
                menus[i].append(mi)
                mi.connect('activate', self.menuitem_response, items[i][j])        
        for ii in menuitems:
            self.mb.append(ii)      
    #------------------------------------------------------------------------------------------
    def menuitem_response(self, widget, string):
    
        if string == 'Save As':
            self.saveAs()
        elif string == 'Load setting':
            self.readSetting()
        elif string == 'Create cylinder':
            self.createCylinder()
        elif string == 'Create box':
            self.createBox()
        elif string == 'Scale':
            self.surfaceScale()
        elif string == 'Translate':
            self.surfaceTranslate()
        elif string == 'Split solids':
            self.splitSolids()
        elif string == 'Merge files':
            self.mergeSTLs()
        elif string == 'surfaceAutoPatch':
            self.surfaceAutoPatch()
        elif string == 'paraFoam':
            os.system('paraFoam -case ' + self.caseDir + '&')
        elif string == 'paraFoam -builtin':
            os.system('paraFoam -builtin -case ' + self.caseDir + '&')
        elif string == 'Snappy User Guide':
            os.system('evince ' + self.installpath + '/Help/Baram-snappy-v6-usersGuide.pdf &')
        elif string == 'Snappy Tutorial':
            os.system('evince ' + self.installpath + '/Help/Baram-snappy-v6-tutorial.pdf &')
        elif string == 'View/Edit setting':
            generalClass(self).indentXML(self.root)
            self.tree.write(self.caseDir + '/setup_snappy.xml') 
            self.editXMLSnappy()
    #----------------------------------------------------------------------------------------            
    def makeTerminal(self):

        if os.path.isfile('/usr/bin/xterm'): # by Gill
            xterm='/usr/bin/xterm'
        else:
            xterm='xterm'
                
        self.terminalSocket = gtk.Socket()
        self.shellbox.add(self.terminalSocket)
        
        socketID = str(self.terminalSocket.get_id())        
        fontSize = '15'
        font = "xterm*font: *-fixed-medium-*-*-" + fontSize + "-*"
        
        os.system("xterm -into " + socketID + " -xrm '" + font + "' -bg white -fg black -e 'tty>" + self.caseDir + "/tty;read'& \n echo $! > " + self.caseDir + '/xtermPID')
        time.sleep(0.5) 
        
        while(not os.path.isfile(self.caseDir + '/tty')):
            pass
        
        with open(self.caseDir + '/tty','r') as f:
            rr = f.read()
            ss = rr.split('/')
            self.termTTY = ss[-1].strip()

        with open(self.caseDir + '/xtermPID','r') as f:
            rr = f.read()
            self.xtermPID = rr.strip()
    #----------------------------------------------------------------------------------------    
    def makeToolbar(self):

        toolbox, logo, buttonDic, self.toolItems = self.STY.toolbarStyleSnappy()
        self.toolbarbox.pack_start(toolbox, True, True, 0)
        self.toolbarbox.pack_end(logo, False, False, 1)
        for ii in buttonDic.keys():
            buttonDic[ii].connect('clicked', self.runToolbar, ii)
    #----------------------------------------------------------------------------------------    
    def makeToolbar_view(self):
            
        toolbox1, buttonDic1 = self.STY.toolbarViewStyle()
        self.viewToolbox.pack_start(toolbox1, True, True, 0)        
        for ii in buttonDic1.keys():
            buttonDic1[ii].connect('clicked', self.runToolbar, ii)      
    #------------------------------------------------------------------------------------------------------
    def runToolbar(self, widget, text):
      
        if text == 'New Project':self.newcase()
        elif text == 'Open Project':self.readcase()
        elif text == 'paraFoam':os.system('paraFoam -case ' + self.caseDir + '&')
        elif text == 'Stop':self.stop()
        elif text == 'Reset':self.resetSize()
        elif text == 'view dir. +X':self.resetview([-1,0,0], [0,0,0], [0,0,1])
        elif text == 'view dir. -X':self.resetview([1,0,0], [0,0,0], [0,0,1])
        elif text == 'view dir. +Y':self.resetview([0,-1,0], [0,0,0], [0,0,1])
        elif text == 'view dir. -Y':self.resetview([0, 1, 0], [0,0,0], [0,0,1])
        elif text == 'view dir. +Z':self.resetview([0,0,-1], [0,0,0], [0,1,0])
        elif text == 'view dir. -Z':self.resetview([0, 0, 1], [0,0,0], [0,1,0])
        elif text == 'Surface culling':self.surfaceCulling()        
        elif text == 'Exit':self.closedialog()
        elif text == 'Baram':self.toBaram()
        else:self.chfreepanel(text)       
    #------------------------------------------------------------------------------------------
    def setfolder(self):
    
        WFILE = writeFileClassSnappy(self.caseDir)
        
        if not os.path.isdir(self.caseDir + '/0'):
            os.system('mkdir ' + self.caseDir + '/0')
        if not os.path.isdir(self.caseDir + '/constant'):
            os.system('mkdir ' + self.caseDir + '/constant')
        if not os.path.isdir(self.caseDir + '/system'):
            os.system('mkdir ' + self.caseDir + '/system')
        if not os.path.isdir(self.caseDir + '/system/settings'):
            os.system('mkdir ' + self.caseDir + '/system/settings')
        if not os.path.isdir(self.caseDir + '/logfiles'):
            os.system('mkdir ' + self.caseDir + '/logfiles')

        if not os.path.isfile(self.caseDir + '/system/fvSolution'):
            WFILE.basicfvsolution()
        if not os.path.isfile(self.caseDir + '/system/fvSchemes'):
            WFILE.basicfvschemes()
        WFILE.basiccontroldict()                     
    #------------------------------------------------------------------------------------------
    def chfreepanel(self, whatitem):        
                
        if whatitem == 'paraFoam':
            return
        
        self.nowPanel = whatitem

        for ii in self.freevbox.get_children():
            self.freevbox.remove(ii)
        
        if whatitem == 'Geometry':
            self.cadPart()
            self.freevbox.add(self.cadbox)
            self.resetRenderer()
        elif whatitem == 'blockMesh':        
            self.FREEBLOCKMESH = freepanelBlockMeshClass(self)
            self.blockbox = self.FREEBLOCKMESH.blockMesh()
            self.freevbox.add(self.blockbox)
        elif whatitem == 'castellate mesh':
            self.FREECASTEL = freepanelCastellateClass(self)
            self.castelbox = self.FREECASTEL.castellate()
            self.freevbox.add(self.castelbox)
            self.resetRenderer()
        elif whatitem == 'snap mesh':
            self.FREESNAP = freepanelSnapClass(self)
            self.snapbox = self.FREESNAP.snapPanel()
            self.freevbox.add(self.snapbox)
        elif whatitem == 'add layer':
            self.FREELAYER = freepanelLayerClass(self)
            self.layerbox = self.FREELAYER.addLayer()
            self.freevbox.add(self.layerbox)
        elif whatitem == 'advanced':
            self.FREEADVANCED = freepanelAdvancedClass(self)
            self.advbox = self.FREEADVANCED.advanced()
            self.freevbox.add(self.advbox)
        elif whatitem == 'Run SnappyHexMesh':
            self.FREERUNSNAPPY = freepanelRunSnappyClass(self)
            self.runbox = self.FREERUNSNAPPY.runSnappy()
            self.freevbox.add(self.runbox)
        elif whatitem == 'Mesh manipulation':
            self.FREEMESH = freepanelMeshSnappyClass(self)
            self.meshbox, self.meshBDbox = self.FREEMESH.mesh()
            self.freevbox.add(self.meshbox)
        self.window.show_all()
        
        if whatitem == 'blockMesh': 
            if not os.path.isdir(self.caseDir + '/constant/triSurface'):
                generalClass(self).makeDialog('Warning!!!! There is not STL file.')
    #-----------------------------------------------------------------------------------------
    def newcase(self):
    
        GEN = generalClass(self)
        returnedname = GEN.newcase()
        
        if returnedname != 'None':
            self.caseDir = returnedname
            self.window.set_title('Baram-snappy-v6.4     ' + self.caseDir)
            for ii in self.freevbox.get_children():
                self.freevbox.remove(ii)

            self.setfolder()
            self.stlnames = []
            self.loadXML()
            self.cadPart()
            self.freevbox.add(self.cadbox)

            self.window.show_all()
            
            self.renderer.RemoveAllViewProps()
            self.gvtk.Initialize()                               
            
            return self.caseDir
    #-------------------------------------------------------------------------------------------------    
    def readcase(self):
    
        GEN = generalClass(self)
        returnedname = GEN.readcase()
        
        if returnedname != 'None':
            self.caseDir = returnedname
            self.setfolder()
            self.loadXML()            
            
            self.stlnames = []
            for ii in self.stl:
                self.stlnames.append(ii.tag)
                
            self.boxActorDict = {}
            for ii in self.box:
                boxActor = self.VTK_All.showBoxNew(ii.find('minx').text, ii.find('miny').text, ii.find('minz').text,
                    ii.find('maxx').text, ii.find('maxy').text, ii.find('maxz').text)
                self.boxActorDict[ii.tag] = boxActor                
                    
            self.window.set_title('Baram-snappy-v6.4     ' + self.caseDir)
            for ii in self.freevbox.get_children():
                self.freevbox.remove(ii)
            self.cadPart()
            self.freevbox.add(self.cadbox)

            self.window.show_all()
            self.notebook.set_current_page(0) 
            
            self.resetRenderer()         
    #------------------------------------------------------------------------------------------           
    def saveAs(self):

        dialog = gtk.FileChooserDialog("Save..", None, gtk.FILE_CHOOSER_ACTION_SAVE,
                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        dialog.set_default_response(gtk.RESPONSE_OK)
        dialog.set_current_name("Untitled")
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            clonename = dialog.get_filename()
        elif response == gtk.RESPONSE_CANCEL:
            clonename = None
        dialog.destroy()

        if os.path.isfile(clonename):
            generalClass(self).makeDialog('Error!!! "' + clonename + '" file exist.')
            return
          
        if clonename:
            os.system('mkdir ' + clonename)
            
            tf = os.access(clonename,os.W_OK)
            if tf == False:
                generalClass(self).makeDialog('Error! Permission denied for ' + clonename)
                return None 

            temp = expanduser("~") + '/.OpenFOAM/Baram_snappy_temporary'            
            if self.caseDir == temp or self.caseDir[:-1] == temp:
                os.system('rm -r ' + clonename)
                os.system('mv ' + self.caseDir + ' ' + clonename)
            else:
                os.system('cp -r ' + self.caseDir + '/* ' + clonename + '/')                
        
            self.caseDir = clonename
            self.window.set_title(self.ver + '     ' + self.caseDir)

            self.window.show_all()           
    # -------------------------------------------------------------------------
    def stop(self): 
        
        runPIDFile = self.caseDir + '/logfiles/runSnappyPID'
        if os.path.isfile(runPIDFile):
            runPID = open(runPIDFile, 'r').read()                        
            if generalClass(self).isRunning(int(runPID)):
                os.system('kill -9 ' + runPID)        
                os.system("kill -9 `ps |grep snappyHexMesh|awk '{print $1}'`")
                os.system('echo  > /dev/pts/' + self.termTTY)
                os.system('echo Process is stopped -------------------------- > /dev/pts/' + self.termTTY)
    #------------------------------------------------------------------------------------------
    def surfaceScale(self):
        SURF = surfaceClass(self)
        SURF.surfaceScale()          

    def surfaceTranslate(self):
        SURF = surfaceClass(self)
        SURF.surfaceTranslate()
          
    def splitSolids(self):
        SURF = surfaceClass(self)
        SURF.splitSolids()  
        
    def mergeSTLs(self):
        SURF = surfaceClass(self)
        SURF.mergeSTLs()
        
    def surfaceAutoPatch(self):
        SURF = surfaceClass(self)
        SURF.surfaceAutoPatch()
    #----------------------------------------------------------------------------------------------------
    def resetview(self, viewposition, target, up):
    
        camera = self.renderer.GetActiveCamera()
        camera.SetPosition(viewposition[0], viewposition[1], viewposition[2])
        camera.SetFocalPoint(target[0], target[1], target[2])
        camera.SetViewUp(up[0], up[1], up[2])
        self.renderer.SetActiveCamera(camera)
        self.renderer.ResetCamera()
        self.gvtk.Initialize()
    #----------------------------------------------------------------------------------------------------
    def surfaceCulling(self):
    
        self.culling = self.STY.surfaceCulling(self.renderer, self.culling)
        self.gvtk.Initialize() 
    #----------------------------------------------------------------------------------------------------
    def resetSize(self):
    
        self.renderer.ResetCamera()
        self.gvtk.Initialize()
    #--------------------------------------------------------------------------------------
    def chbg(self, widget):
    
        self.STY.changeBackgroundColor(self.renderer, widget.child.get_text(), self.rgb, 
                                       self.displayMode, self.backgroundColor)    
        self.backgroundColor = widget.child.get_text()
        self.gvtk.Initialize()
    #---------------------------------------------------------------------------------------------------------            
    def changeDisplayMode(self, widget):
    
        self.displayMode = self.discombo.child.get_text()
        num = self.renderer.GetActors().GetNumberOfItems()
                                
        for i in range(num):
            if self.displayMode == 'SurfaceEdge':            
                self.renderer.GetActors().GetItemAsObject(i).GetProperty().EdgeVisibilityOn()
                self.renderer.GetActors().GetItemAsObject(i).GetProperty().SetRepresentation(2)
            elif self.displayMode == 'Surface':
                self.renderer.GetActors().GetItemAsObject(i).GetProperty().EdgeVisibilityOff()
                self.renderer.GetActors().GetItemAsObject(i).GetProperty().SetRepresentation(2)
            elif self.displayMode == 'Wireframe':
                self.renderer.GetActors().GetItemAsObject(i).GetProperty().SetRepresentationToWireframe()
        self.gvtk.Initialize()
    #------------------------------------------------------------------------------------------------------                  
    def closedialog(self):
    
        GEN = generalClass(self)
        res = GEN.makeDialog_question('Exit this program ?')
        if res == -9:
            return
        else:
            size = self.window.get_size()
            GEN.pickleDump(self.home + '/.OpenFOAM/windowSize', size)
            gtk.main_quit() 

            temp = expanduser("~") + '/.OpenFOAM/Baram_temporary'
            if self.caseDir == temp or self.caseDir[:-1] == temp:
                os.system('rm -rf ' + self.caseDir)
                
            if generalClass(self).isRunning(int(self.xtermPID)):
                os.system('kill -9 ' + self.xtermPID)

    #--------------------------------------------------------------------------------------------
    def toBaram(self):

        def openSolver(widget):                       
            gtk.main_quit() 
            
            opt = combo.child.get_text()
            if opt == 'incompressible':
                os.system('cd ' + self.installpath + ' && ./Baram.py ' + self.caseDir + ' fromSnappy&')
            elif opt == 'compressible':
                os.system('cd ' + self.installpath + ' && ./Baram.py -aero ' + self.caseDir + ' fromSnappy&')
            win.destroy()

            if generalClass(self).isRunning(int(self.xtermPID)):
                os.system('kill -9 ' + self.xtermPID)
                                
        win, combo, button = self.STY.selectMode(self.window)        
        button.connect('clicked', openSolver)
        win.show_all() 
    #--------------------------------------------------------------------------------------------
    def leftButtonPressEvent(self, obj, event):       
           
        if self.gvtk.bCtrl == False:
            return
            
        clickPos = self.gvtk.GetEventPosition()
        self.cellPicker.Pick(clickPos[0], clickPos[1], 0, self.renderer)
        selectedActor = self.cellPicker.GetActor()

        if self.nowPanel != 'Geometry':
            return
        
        if selectedActor:           
            if selectedActor in self.Actors:
                for i in range(len(self.Actors)):
                    if selectedActor == self.Actors[i]:
                        self.Actors[i].GetProperty().SetColor(0, 0, 1)
                        path = (0, i,)
                        self.treeview.set_cursor(path)                                   
                    else:
                        self.Actors[i].GetProperty().SetColor(1, 1, 1)  
            else:
                for i in range(len(self.box)):
                    boxname = self.box[i].tag
                    if selectedActor == self.boxActorDict[boxname]:
                        self.boxActorDict[boxname].GetProperty().SetColor(0, 0, 1)
                        path = (1, i,)
                        self.treeview.set_cursor(path) 
                    else:
                        self.boxActorDict[boxname].GetProperty().SetColor(1, 1, 1)
    #--------------------------------------------------------------------------------------------
    # CAD Panel
    #--------------------------------------------------------------------------------------------
    def cadPart(self):
    
        self.cadbox = gtk.VBox()          
        
        swin = gtk.ScrolledWindow()
        swin.set_border_width(0)
        swin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
        swin.set_size_request(400, -1)
        
        self.underbox = gtk.VBox(False, 0)
        self.cadbox.pack_start(swin, True, True, 0)
        self.cadbox.pack_end(self.underbox, False, False, 0)
        
        ebox = gtk.EventBox()
        ebox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse('white'))
        swin.add_with_viewport(ebox)
        self.cadMainbox = gtk.VBox(False, 0)
        ebox.add(self.cadMainbox)

        self.getListStore()
        #-------------------------------------------------------------------------------------       
        self.treestore = gtk.TreeStore(gtk.gdk.Pixbuf, str, str, str, str)
        
        self.treeview = gtk.TreeView(self.treestore)
        
        self.treeview.set_headers_visible(False)
        self.treeview.set_enable_tree_lines(True)

        self.treeview.connect('cursor-changed', self.selected, self.treeview)
                
        self.tvcolumn0 = gtk.TreeViewColumn(None)
        self.tvcolumn1 = gtk.TreeViewColumn(None)
        
        self.treeview.append_column(self.tvcolumn0)
        self.treeview.append_column(self.tvcolumn1)

        self.renbuf = gtk.CellRendererPixbuf()
        self.ren0 = gtk.CellRendererText()
        self.ren1_0 = gtk.CellRendererText()
        self.ren1_1 = gtk.CellRendererText()
        self.ren1_2 = gtk.CellRendererText()
        self.ren2 = gtk.CellRendererCombo()                

        self.ren1_0.set_fixed_size(40, -1)
        self.ren1_1.set_fixed_size(40, -1)
        self.ren1_2.set_fixed_size(40, -1)
        
        self.ren1_0.set_property("editable", True)
        self.ren1_1.set_property("editable", True)
        self.ren1_2.set_property("editable", True)
        self.ren2.set_property("editable", True)
        self.ren2.set_property('text-column', 0)
        
        self.ren1_0.connect('edited', self.changeValue, 'x')
        self.ren1_1.connect('edited', self.changeValue, 'y')
        self.ren1_2.connect('edited', self.changeValue, 'z')
        self.ren2.connect('edited', self.changeValue, 'x')
        
        self.tvcolumn0.pack_start(self.renbuf, False)
        self.tvcolumn0.pack_start(self.ren0, False)
        self.tvcolumn0.set_attributes(self.renbuf, pixbuf = 0)
        self.tvcolumn0.set_attributes(self.ren0, text = 1)        
        
        self.tvcolumn1.pack_start(self.ren1_0, True)
        self.tvcolumn1.pack_start(self.ren1_1, True) 
        self.tvcolumn1.pack_start(self.ren1_2, True)
        self.tvcolumn1.pack_start(self.ren2, True)         
        self.tvcolumn1.set_cell_data_func(self.ren1_0, self.func)
        self.tvcolumn1.set_cell_data_func(self.ren2, self.func)

        self.makeTreeStore()
        self.treeview.expand_all()
        self.cadMainbox.pack_start(self.treeview, True, True, 0)             
        
        # underbox
        textview = gtk.TextView()
        self.rangebuffer = textview.get_buffer()
        vbox = gtk.VBox(False, 5)
        self.underbox.pack_start(vbox, False, False, 0)
        vbox.pack_start(textview,False, False, 0)
        # button     
        self.addSTLB = gtk.Button('Add STL')
        self.addSTLB.set_size_request(150, 28)
        self.addSTLB.connect('clicked', self.addSTL)
        
        self.delSTLB = gtk.Button('Delete STL')
        self.delSTLB.set_size_request(150, 28)
        self.delSTLB.connect('clicked', self.delSTL)
        
        self.addBoxB = gtk.Button('Add Searchable Object')
        self.addBoxB.set_size_request(150, 28)
        self.addBoxB.connect('clicked', self.addObject)
        
        self.delBoxB = gtk.Button('Delete Searchable Object')
        self.delBoxB.set_size_request(150, 28)
        self.delBoxB.connect('clicked', self.delBox)
        
        hbox = gtk.HBox(False, 0)
        self.underbox.pack_start(hbox, False, False, 0)
        hbox.pack_start(self.addSTLB, True, True, 0)
        hbox.pack_end(self.delSTLB, True, True, 0)
        
        hbox = gtk.HBox(False, 0)
        self.underbox.pack_start(hbox, False, False, 0)
        hbox.pack_start(self.addBoxB, True, True, 0)
        hbox.pack_start(self.delBoxB, True, True, 0)
    #--------------------------------------------------------------------------------------------------------- 
    def makeTreeStore(self):
    
        self.treestore.clear()
        
        row_stl = self.treestore.append(None, [self.pixbuf_main, 'Geometry', None, None, None])
        row_box = self.treestore.append(None, [self.pixbuf_main, 'Searchable objects', None, None, None])
        row_feature = self.treestore.append(None, [self.pixbuf_main, 'Feature', None, None, None]) 
                
        for ii in self.stlnames:
            name = self.stl.find(ii)
            row_name = self.treestore.append(row_stl, [self.pixbuf_first, ii, name.find('type').text, None, None])
            if name.find('type').text == 'cellZone':
                self.treestore.append(row_name, [self.pixbuf_second, 'mode', name.find('mode').text, None, None])
                self.treestore.append(row_name, [self.pixbuf_second, 'level', name.find('level').text, None, None])
                if name.find('mode').text == 'distance':
                    self.treestore.append(row_name, [self.pixbuf_second, 'distance', name.find('distance').text, None, None])
            
        num = len(self.box)
        for i in range(num):             
            box = self.treestore.append(row_box, [self.pixbuf_first, self.box[i].tag, None, None, None])
            if self.box[i].find('type').text == 'box':
                self.treestore.append(box, [self.pixbuf_second, 'min. x y z', None, None, None])
                self.treestore.append(box, [self.pixbuf_second, 'max. x y z', None, None, None])
            elif self.box[i].find('type').text == 'cylinder':
                self.treestore.append(box, [self.pixbuf_second, 'center', None, None, None])
                self.treestore.append(box, [self.pixbuf_second, 'height', None, None, None])
                self.treestore.append(box, [self.pixbuf_second, 'radius', None, None, None])
                self.treestore.append(box, [self.pixbuf_second, 'axis', None, None, None])
            elif self.box[i].find('type').text == 'sphere':
                self.treestore.append(box, [self.pixbuf_second, 'center', None, None, None])
                self.treestore.append(box, [self.pixbuf_second, 'radius', None, None, None])

        self.treestore.append(row_feature, [self.pixbuf_first, 'feature angle', None, None, None])
        self.treestore.append(row_feature, [self.pixbuf_first, 'keep nonManifoldEdges', None, None, None])
        self.treestore.append(row_feature, [self.pixbuf_first, 'keep openEdges', None, None, None])

        self.treeview.expand_all()        
    #--------------------------------------------------------------------------------------------------------- 
    def func(self, column, cell, model, iter):
    
        value0 = model.get_value(iter, 1)
        path = model.get_path(iter)
        
        if value0 == 'Geometry' or value0 == 'Searchable objects' or value0 == 'Feature':
            self.ren1_0.set_property('visible', 0) 
            self.ren1_1.set_property('visible', 0) 
            self.ren1_2.set_property('visible', 0)
            self.ren2.set_property('visible', 0)
            return
            
        if path[0] == 0:
            self.ren1_0.set_property('visible', 0)
            self.ren1_1.set_property('visible', 0)
            self.ren1_2.set_property('visible', 0)
            self.ren2.set_property('visible', 1)
            if value0 in self.stlnames:
                name = self.stl.find(value0)
                self.ren2.set_property('text', name.find('type').text)
                self.ren2.set_property('model', self.liststore_types)
                self.ren2.set_property('has-entry', False)
            else:
                pp = (path[0], path[1],)               
                nn = self.treestore[pp][1]
                name = self.stl.find(nn)
                if value0 == 'mode':               
                    self.ren2.set_property('text', name.find('mode').text)
                    self.ren2.set_property('model', self.liststore_zoneMode)
                    self.ren2.set_property('has-entry', False)
                elif value0 == 'level':
                    self.ren2.set_property('text', name.find('level').text)
                    self.ren2.set_property('model', None)
                    self.ren2.set_property('has-entry', True) 
                elif value0 == 'distance':
                    self.ren2.set_property('text', name.find('distance').text)
                    self.ren2.set_property('model', None)
                    self.ren2.set_property('has-entry', True)            
        elif path[0] == 1:
            pp = (path[0], path[1],)               
            nn = self.treestore[pp][1]
            name = self.box.find(nn)
            if value0[:6] == 'object':
                self.ren2.set_property('visible', 1)
                self.ren1_0.set_property('visible', 0)
                self.ren1_1.set_property('visible', 0)
                self.ren1_2.set_property('visible', 0)
                self.ren2.set_property('text', name.find('type').text)
                self.ren2.set_property('model', self.liststore_objects)
                self.ren2.set_property('has-entry', False)
            elif value0 == 'min. x y z':
                self.ren2.set_property('visible', 0)
                self.ren1_0.set_property('visible', 1)
                self.ren1_0.set_property('text', name.find('minx').text)
                self.ren1_1.set_property('visible', 1)
                self.ren1_1.set_property('text', name.find('miny').text)
                self.ren1_2.set_property('visible', 1)
                self.ren1_2.set_property('text', name.find('minz').text) 
            elif value0 == 'max. x y z':
                self.ren2.set_property('visible', 0)
                self.ren1_0.set_property('visible', 1)
                self.ren1_0.set_property('text', name.find('maxx').text)
                self.ren1_1.set_property('visible', 1)
                self.ren1_1.set_property('text', name.find('maxy').text)
                self.ren1_2.set_property('visible', 1)
                self.ren1_2.set_property('text', name.find('maxz').text) 
            if value0 == 'axis':
                self.ren2.set_property('visible', 1)
                self.ren1_0.set_property('visible', 0)
                self.ren1_1.set_property('visible', 0)
                self.ren1_2.set_property('visible', 0)
                self.ren2.set_property('text', name.find('axis').text)
                self.ren2.set_property('model', self.liststore_axis)
                self.ren2.set_property('has-entry', False)
            elif value0 == 'center':
                self.ren2.set_property('visible', 0)
                self.ren1_0.set_property('visible', 1)
                self.ren1_0.set_property('text', name.find('centerx').text)
                self.ren1_1.set_property('visible', 1)
                self.ren1_1.set_property('text', name.find('centery').text)
                self.ren1_2.set_property('visible', 1)
                self.ren1_2.set_property('text', name.find('centerz').text) 
            elif value0 == 'radius':
                self.ren2.set_property('visible', 0)
                self.ren1_0.set_property('visible', 1)
                self.ren1_0.set_property('text', name.find('radius').text)
                self.ren1_1.set_property('visible', 0)
                self.ren1_2.set_property('visible', 0)
            elif value0 == 'height':
                self.ren2.set_property('visible', 0)
                self.ren1_0.set_property('visible', 1)
                self.ren1_0.set_property('text', name.find('height').text)
                self.ren1_1.set_property('visible', 0)
                self.ren1_2.set_property('visible', 0)
        elif path[0] == 2:
            if value0 == 'feature angle':
                self.ren1_0.set_property('visible', 1)
                self.ren1_1.set_property('visible', 0)
                self.ren1_2.set_property('visible', 0)
                self.ren2.set_property('visible', 0)
                self.ren1_0.set_property('text', self.cad.find('featureAngle').text)
            elif value0 == 'keep nonManifoldEdges':
                self.ren1_0.set_property('visible', 0)
                self.ren1_1.set_property('visible', 0)
                self.ren1_2.set_property('visible', 0)
                self.ren2.set_property('visible', 1)
                self.ren2.set_property('text', self.cad.find('nonManifoldEdges').text)
                self.ren2.set_property('model', self.liststore_yesno)
                self.ren2.set_property('has-entry', False)            
            elif value0 == 'keep openEdges':
                self.ren1_0.set_property('visible', 0)
                self.ren1_1.set_property('visible', 0)
                self.ren1_2.set_property('visible', 0)
                self.ren2.set_property('visible', 1)
                self.ren2.set_property('text', self.cad.find('openEdges').text)
                self.ren2.set_property('model', self.liststore_yesno)
                self.ren2.set_property('has-entry', False)
    #---------------------------------------------------------------------------------------------------------        
    def changeValue(self, widget, path, what, xyz):
    
        treeiter = self.treestore.get_iter(path)
        value0 = self.treestore.get_value(treeiter, 1)
        old = self.treestore[path][2]      

        if path[0] == '0':
            self.treestore[path][2] = what
            if value0 in self.stlnames:
                name = self.stl.find(value0)
                name.find('type').text = what
                
                if what == 'cellZone':
                    ind = self.stlnames.index(value0)
                    self.Actors[ind].GetProperty().SetRepresentationToWireframe()
                    self.gvtk.Initialize()
                else:
                    if old == 'cellZone' and what != 'cellZone':
                        ind = self.stlnames.index(value0)
                        self.Actors[ind].GetProperty().SetRepresentation(2)
                        self.gvtk.Initialize()
                
            else:
                pp = (int(path[0]), int(path[2]),)               
                nn = self.treestore[pp][1]
                name = self.stl.find(nn)
                name.find(value0).text = what
                
            self.makeTreeStore()
        elif path[0] == '1':        
            pp = (int(path[0]), int(path[2]),)               
            nn = self.treestore[pp][1]
            name = self.box.find(nn)
            
            if value0[:6] == 'object':
                name.clear()
                if what == 'box':
                    ET.SubElement(name, 'type').text = 'box'
                    ET.SubElement(name, 'minx').text = '0'
                    ET.SubElement(name, 'miny').text = '0'
                    ET.SubElement(name, 'minz').text = '0'
                    ET.SubElement(name, 'maxx').text = '1'
                    ET.SubElement(name, 'maxy').text = '1'
                    ET.SubElement(name, 'maxz').text = '1' 
                elif what == 'cylinder':
                    ET.SubElement(name, 'type').text = 'cylinder'
                    ET.SubElement(name, 'centerx').text = '0'
                    ET.SubElement(name, 'centery').text = '0'
                    ET.SubElement(name, 'centerz').text = '0'
                    ET.SubElement(name, 'radius').text = '1'
                    ET.SubElement(name, 'height').text = '1'
                    ET.SubElement(name, 'axis').text = 'x' 
                elif what == 'sphere':
                    ET.SubElement(name, 'type').text = 'sphere'
                    ET.SubElement(name, 'centerx').text = '0'
                    ET.SubElement(name, 'centery').text = '0'
                    ET.SubElement(name, 'centerz').text = '0'
                    ET.SubElement(name, 'radius').text = '1' 

                self.makeTreeStore()                
            
            elif value0 == 'min. x y z':
                if xyz == 'x':
                    name.find('minx').text = what
                elif xyz == 'y':
                    name.find('miny').text = what
                elif xyz == 'z':
                    name.find('minz').text = what
            elif value0 == 'max. x y z':
                if xyz == 'x':
                    name.find('maxx').text = what
                elif xyz == 'y':
                    name.find('maxy').text = what
                elif xyz == 'z':
                    name.find('maxz').text = what               
            elif value0 == 'center':
                if xyz == 'x':
                    name.find('centerx').text = what
                elif xyz == 'y':
                    name.find('centery').text = what
                elif xyz == 'z':
                    name.find('centerz').text = what                    
            elif value0 == 'radius':
                name.find('radius').text = what
            elif value0 == 'height':
                name.find('height').text = what
            elif value0 == 'axis':
                name.find('axis').text = what
                                    
            self.renderer.RemoveActor(self.boxActorDict[nn])
            self.createBoxActor(name)
        elif path[0] == '2':        
            if value0 == 'feature angle':
                self.cad.find('featureAngle').text = what
            elif value0 == 'keep nonManifoldEdges':
                self.cad.find('nonManifoldEdges').text = what
            elif value0 == 'keep openEdges':
                self.cad.find('openEdges').text = what
                        
        generalClass(self).indentXML(self.root) 
        self.tree.write(self.caseDir + '/setup_snappy.xml')            
    #------------------------------------------------------------
    def addSTL(self, widget):
    
        # Receive File List
        if not os.path.isdir(self.caseDir + '/constant/triSurface'):
            os.system('mkdir ' + self.caseDir + '/constant/triSurface')        
        
        stlfilelist = generalClass(self).selSTL()
        
        asciifiles = []
        
        for ii in stlfilelist:
            ab = generalClass(self).findAsciiBinary(ii)
            if ab == 'ascii':
                asciifiles.append(ii)
            else:            
                outputpath = self.caseDir + '/constant/triSurface'
                generalClass(self).binaryToAscii(ii, outputpath)
                basename = os.path.basename(ii)
                self.stlnames.append(basename[:-4])                        
        
        for ii in asciifiles:
            numRegion = len(generalClass(self).findSTLRegion(ii)) 
            if numRegion > 1:
                regions = self.makeSolidSTLFile(ii)
                for jj in regions:
                    self.stlnames.append(jj)
            elif numRegion == 1:
                nameonly = os.path.basename(ii)               
                os.system('cp ' + ii + ' ' + self.caseDir + '/constant/triSurface/' + nameonly) 
                self.stlnames.append(nameonly[:-4])
            else:
                return
        
        self.createCadXML()
        self.makeTreeStore()
        
        self.notebook.set_current_page(0)           

        # Model Display        
        selectedstl = []
        for i in range(len(self.stlnames)):
            selectedstl.append(self.caseDir + '/constant/triSurface/' + self.stlnames[i] + '.stl')
                
        self.renderer.RemoveAllViewProps()
        self.Actors = self.VTK_All.showSTLMultiForSnappy(selectedstl)
        for ii in self.Actors:
            self.renderer.AddActor(ii)
        self.gvtk.Initialize()
        self.resetSize()
        
        generalClass(self).indentXML(self.root) 
        self.tree.write(self.caseDir + '/setup_snappy.xml')               
    #------------------------------------------------------------
    def delSTL(self, widget):
    
        if self.selectedPatch == None:
            generalClass(self).makeDialog('Select STL Region first!!!')
            return
        
        os.system('rm ' + self.caseDir + '/constant/triSurface/' + self.selectedPatch + '.stl')
        self.stlnames.remove(self.selectedPatch)
        
        em = self.stl.find(self.selectedPatch)
        self.stl.remove(em)
        
        em = self.surfaceLevel.find(self.selectedPatch)
        self.surfaceLevel.remove(em)
        
        em = self.featureLevel.find(self.selectedPatch)
        self.featureLevel.remove(em)
        
        self.makeTreeStore()     
        
        selectedstl = []
        for i in range(len(self.stlnames)):
            selectedstl.append(self.caseDir + '/constant/triSurface/' + self.stlnames[i] + '.stl')
            
        self.renderer.RemoveAllViewProps()
        self.Actors = self.VTK_All.showSTLMultiForSnappy(selectedstl)
        for ii in self.Actors:
            self.renderer.AddActor(ii)
        self.gvtk.Initialize()

        generalClass(self).indentXML(self.root) 
        self.tree.write(self.caseDir + '/setup_snappy.xml')    
    #------------------------------------------------------------
    def addObject(self, widget):
        
        newName = self.getNewBoxName()
        box = ET.SubElement(self.box, newName)
        ET.SubElement(box, 'type').text = 'box'
        ET.SubElement(box, 'minx').text = '0'
        ET.SubElement(box, 'miny').text = '0'
        ET.SubElement(box, 'minz').text = '0'
        ET.SubElement(box, 'maxx').text = '1'
        ET.SubElement(box, 'maxy').text = '1'
        ET.SubElement(box, 'maxz').text = '1'        
        self.makeTreeStore()       
        
        self.createBoxActor(box)         

        ET.SubElement(self.boxLevel, newName).text = '1'

        generalClass(self).indentXML(self.root) 
        self.tree.write(self.caseDir + '/setup_snappy.xml')
    #------------------------------------------------------------
    def delBox(self, widget):

        if self.selectedBox == None:
            generalClass(self).makeDialog('Select Searchable object first!!!')
            return
            
        em = self.box.find(self.selectedBox)
        self.box.remove(em)
        
        em = self.boxLevel.find(self.selectedBox)
        self.boxLevel.remove(em)
        
        self.renderer.RemoveActor(self.boxActorDict[em.tag])
        del self.boxActorDict[em.tag]
        
        self.makeTreeStore()
        self.gvtk.Initialize()  

        generalClass(self).indentXML(self.root) 
        self.tree.write(self.caseDir + '/setup_snappy.xml')
    #-------------------------------------------------------------------------------------------
    def createBoxActor(self, boxEm):

        if boxEm.find('type').text == 'box':
            minx = boxEm.find('minx').text
            miny = boxEm.find('miny').text
            minz = boxEm.find('minz').text
            maxx = boxEm.find('maxx').text
            maxy = boxEm.find('maxy').text
            maxz = boxEm.find('maxz').text
            boxActor = self.VTK_All.showBoxNew(minx, miny, minz, maxx, maxy, maxz)
        elif boxEm.find('type').text == 'sphere':
            centerx = boxEm.find('centerx').text
            centery = boxEm.find('centery').text
            centerz = boxEm.find('centerz').text
            radius = boxEm.find('radius').text
            actor = self.VTK_All.showSphere(centerx, centery, centerz, radius)            
            boxActor = actor[0]
            props = vtk.vtkProperty()
            props.SetColor(1, 0, 0)
            boxActor.SetProperty(props)
            boxActor.GetProperty().SetRepresentationToWireframe()
        elif boxEm.find('type').text == 'cylinder':
            height = boxEm.find('height').text
            axis = boxEm.find('axis').text
            centerx = boxEm.find('centerx').text
            centery = boxEm.find('centery').text
            centerz = boxEm.find('centerz').text
            radius = boxEm.find('radius').text
            actor = self.VTK_All.showCone(centerx, centery, centerz, height, axis, radius, radius)                       
            boxActor = actor[0]
            props = vtk.vtkProperty()
            props.SetColor(1, 0, 0)
            boxActor.SetProperty(props)
            boxActor.GetProperty().SetRepresentationToWireframe()
            
        self.boxActorDict[boxEm.tag] = boxActor
        self.renderer.AddActor(boxActor)
        self.gvtk.Initialize()                         
    #--------------------------------------------------------------------------------------------
    def loadXML(self):
    
        xmlfile = self.caseDir + '/setup_snappy.xml'        
        if not os.path.isfile(xmlfile):
            os.system('cp ' + self.installpath + '/setup_snappy.xml ' + self.caseDir + '/')
            os.system('cp ' + self.installpath + '/setup.xml ' + self.caseDir + '/')
            os.system('cp ' + self.installpath + '/setup_aero.xml ' + self.caseDir + '/')
            
        self.tree = ET.parse(xmlfile)
        self.root = self.tree.getroot()
        self.cad = self.root.find('cad')
        self.stl = self.cad.find('stlregions')
        self.box = self.cad.find('boxes')
        self.castel = self.root.find('castellate')
        self.surfaceLevel = self.castel.find('surfaceLevel')
        self.featureLevel = self.castel.find('featureLevel')
        self.boxLevel = self.castel.find('boxLevel')
        self.layer = self.root.find('layer')
        self.layerPatch = self.layer.find('patches')
    #--------------------------------------------------------------------------------------------
    def createCadXML(self):

        for i in range(len(self.stlnames)):
            try:
                self.stl.find(self.stlnames[i]).tag
            except:            
                regionName = ET.SubElement(self.stl, self.stlnames[i])
                ET.SubElement(regionName, 'type').text = 'wall'
                ET.SubElement(regionName, 'mode').text = 'inside'
                ET.SubElement(regionName, 'level').text = '1'
                ET.SubElement(regionName, 'distance').text = '1e-5'            
                
                ET.SubElement(self.surfaceLevel, self.stlnames[i]).text = '1'
                ET.SubElement(self.featureLevel, self.stlnames[i]).text = '1'
                
                ET.SubElement(self.layerPatch, self.stlnames[i]).text = 'no'
            
        generalClass(self).indentXML(self.root)
        self.tree.write(self.caseDir + '/setup_snappy.xml')
    #---------------------------------------------------------------------------------------------------------                    
    def makeSolidSTLFile(self, filename):
    
        with open(filename, 'r') as f:
            ff = f.readlines()
        
        regionnames = []
        startlines = []
        for i in range(len(ff)):
            if (ff[i][:5]).lower() == 'solid': 
                a1 = ff[i].split(' ')
                regionnames.append(a1[1][:-1])
                startlines.append(i + 1)
        nlines = []
        for k in range(len(regionnames) - 1):
            cc = int(startlines[k + 1]) - int(startlines[k])
            nlines.append(cc)
        nlines.append(len(ff) - int(startlines[-1]) + 1)
                                
        for j in range(len(regionnames)):
            g = open(self.caseDir + '/constant/triSurface/' + regionnames[j] + '.stl','w')
            for ii in range(nlines[j]):
                jj = startlines[j] - 1 + ii
                g.write(ff[jj])
            g.close()    
        
        regionnames.sort()
        
        return regionnames    
    #---------------------------------------------------------------------------------------------
    def getListStore(self):

        types = ['wall', 'patch', 'cellZone', 'baffle', 'internal']
        zoneMode = ['inside', 'outside', 'distance']
        yesno = ['yes', 'no']
        objects = ['box', 'cylinder', 'sphere']
        axis = ['x', 'y', 'z']
        
        self.liststore_types = gtk.ListStore(str)
        for ii in types:
            self.liststore_types.append([ii])
        
        self.liststore_zoneMode = gtk.ListStore(str)
        for ii in zoneMode:
            self.liststore_zoneMode.append([ii])
            
        self.liststore_yesno = gtk.ListStore(str)
        for ii in yesno:
            self.liststore_yesno.append([ii])
            
        self.liststore_objects = gtk.ListStore(str)
        for ii in objects:
            self.liststore_objects.append([ii])
            
        self.liststore_axis = gtk.ListStore(str)
        for ii in axis:
            self.liststore_axis.append([ii])
    #---------------------------------------------------------------------------------------------------------             
    def selected(self, widget, treeview):
    
        self.selection = treeview.get_selection()
        tree_model, tree_iter = self.selection.get_selected()
        
        if tree_iter == None:
            return
                    
        path = tree_model.get_path(tree_iter)

        if path[0] == 0:
            if len(path) == 2:
                self.selectedPatch = tree_model.get_value(tree_iter, 1)
                self.highlightColor()
                self.setRange() 
        elif path[0] == 1:
            if len(path) == 2:
                self.selectedBox = tree_model.get_value(tree_iter, 1)
                self.highlightColorBox()
    #-----------------------------------------------------------------------------
    def highlightColor(self):   
     
        if self.selectedPatch != None:
            newind = self.stlnames.index(self.selectedPatch)                       
            for i in range(self.renderer.GetActors().GetNumberOfItems()):
                if i == newind:
                    self.renderer.GetActors().GetItemAsObject(i).GetProperty().SetColor(0, 0, 1)
                else: 
                    self.renderer.GetActors().GetItemAsObject(i).GetProperty().SetColor(1, 1, 1)          
            self.gvtk.Initialize()                         
    #-----------------------------------------------------------------------------
    def highlightColorBox(self):   
     
        if self.selectedBox != None:
            keys = self.boxActorDict.keys()
            for ii in keys:
                if ii == self.selectedBox:
                    self.boxActorDict[ii].GetProperty().SetColor(0, 0, 1)
                else:
                    self.boxActorDict[ii].GetProperty().SetColor(1, 1, 1)        
            self.gvtk.Initialize()  
    #---------------------------------------------------------------------------------------------------------                    
    def setRange(self):
    
        if glob.glob(self.caseDir + '/constant/triSurface/' + self.selectedPatch + '.stl'):        
            stlrange = generalClass(self).getSTLRange(self.caseDir + '/constant/triSurface/' + self.selectedPatch + '.stl')
        else:
            stlrange = generalClass(self).getSTLRange(self.caseDir + '/constant/triSurface/' + self.stlnames[0] + '.stl')
        
        xmin = '%0.6f'%stlrange[0][0]
        ymin = '%0.6f'%stlrange[0][1]
        zmin = '%0.6f'%stlrange[0][2]
        xmax = '%0.6f'%stlrange[1][0]
        ymax = '%0.6f'%stlrange[1][1]
        zmax = '%0.6f'%stlrange[1][2]  
        
        string = '    Range of "'+self.selectedPatch + '"'
        string = string + '\n        Min. : (' + str(xmin) + ' ' + str(ymin) + ' ' + str(zmin) + ') \n'
        string = string + '        Max. : (' + str(xmax) + ' ' + str(ymax) + ' ' + str(zmax) + ')'
        self.rangebuffer.set_text(string)
    #---------------------------------------------------------------------------------------------
    def getNewBoxName(self):

        boxes = []
        for ii in self.box:
            boxes.append(ii.tag)
        nums = []
        for ii in boxes:
            aa = ii.split('-')            
            nums.append(int(aa[1]))
        if nums == []:
            name = 'object-0'
        else:
            name = 'object-' + str(max(nums) + 1)          
        return name          
    #---------------------------------------------------------------------------------------------
    def resetRenderer(self):       
       
        selectedstl = []
        for i in range(len(self.stlnames)):
            selectedstl.append(self.caseDir + '/constant/triSurface/' + self.stlnames[i] + '.stl')
                
        self.renderer.RemoveAllViewProps()
        self.Actors = self.VTK_All.showSTLMultiForSnappy(selectedstl)
        for i in range(len(self.Actors)):
            self.renderer.AddActor(self.Actors[i])
            if self.stl[i].find('type').text == 'cellZone':
                self.Actors[i].GetProperty().SetRepresentationToWireframe()
            
        keys = self.boxActorDict.keys()
        for ii in keys:
            self.boxActorDict[ii].GetProperty().SetRepresentationToWireframe()
            self.renderer.AddActor(self.boxActorDict[ii])
            
        self.gvtk.Initialize()
        self.resetSize()        
    # -----------------------------------------------------------------------------------------------------
    def editXMLSnappy(self):

        def closethis(widget):        
            win.destroy()

        def applythis(widget):        
            self.indentXML(self.root)
            self.tree.write(self.xmlfile) 
            win.destroy()
       
        self.xmlfile = self.caseDir + '/setup_snappy.xml'

        win = gtk.Window(gtk.WINDOW_TOPLEVEL)
        win.set_title("Paned Windows")
        win.connect("destroy", lambda w: win.destroy())
        win.set_border_width(5)
        win.set_size_request(550, 800)
        win.set_transient_for(self.window)
        
        swin = gtk.ScrolledWindow()
        swin.set_border_width(0)
        swin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
        swin.set_size_request(400, -1)

        box = gtk.VBox()        
        win.add(box)
        box.pack_start(swin, True, True, 0)

        ebox = gtk.EventBox()
        ebox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse('white'))
        swin.add_with_viewport(ebox)
        mainbox = gtk.VBox(False, 0)
        ebox.add(mainbox)

        self.treestore1 = gtk.TreeStore(gtk.gdk.Pixbuf, str, str)
        self.makeTreeStore1()                 

        treeview = gtk.TreeView(self.treestore1)
        treeview.set_headers_visible(False)        
        treeview.set_enable_tree_lines(True)
        treeview.set_grid_lines(False)
                
        tvcolumn0 = gtk.TreeViewColumn(None)
        tvcolumn1 = gtk.TreeViewColumn(None)        
        
        treeview.append_column(tvcolumn0)
        treeview.append_column(tvcolumn1)
        
        renbuf = gtk.CellRendererPixbuf()
        ren0 = gtk.CellRendererText()
        ren1 = gtk.CellRendererText()                  

        ren0.set_property('editable', False)
        ren1.set_property('editable', True)
        
        tvcolumn0.pack_start(renbuf, False)
        tvcolumn0.pack_start(ren0, True)
        tvcolumn1.pack_start(ren1, True)        
        
        tvcolumn0.set_attributes(renbuf, pixbuf = 0)
        tvcolumn0.set_attributes(ren0, markup = 1)        
        tvcolumn1.set_attributes(ren1, markup = 2)
        
        ren1.connect('edited', self.changeXML)               
        mainbox.pack_start(treeview, True, True, 0)
        
        btn1 = gtk.Button('Save')
        btn2 = gtk.Button('Close')

        btn1.set_size_request(150, 28)               
        btn2.set_size_request(150, 28)        

        btn1.connect('clicked', applythis)
        btn2.connect('clicked', closethis)
        
        hbox = gtk.HBox(False, 0)
        box.pack_start(hbox, False, False, 0)
        hbox.pack_start(btn1, True, True, 0)
        hbox.pack_start(btn2, True, True, 0)        

        win.show_all()                       
            
    def indentXML(self, elem, level = 0):
        i = "\n" + level*"  "
        if len(elem):
            if not elem.text or not elem.text.strip():
                elem.text = i + "  "
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
            for elem in elem:
                self.indentXML(elem, level+1)
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
        else:
            if level and (not elem.tail or not elem.tail.strip()):
                elem.tail = i
                
    def changeXML(self, widget, path, what):

        treeiter = self.treestore1.get_iter(path)
        value0 = self.treestore1.get_value(treeiter, 1)       
        pathlist = path.split(':')      
        
        tags = [value0]
        iters = [treeiter]
        for i in range(len(pathlist) -1):
            iters.append(self.treestore1.iter_parent(iters[i]))
            tags.append(self.treestore1.get_value(iters[i + 1], 1))
            
        ems = [self.root]
        for i in range(len(tags) - 1):            
            ems.append(ems[i].find(tags[len(tags) - i - 1]))
        ems[-1].find(value0).text = what
       
        self.treestore1[path][2] = what               
                        
    def makeTreeStore1(self):

        for ii in self.root:
            row_main = self.treestore1.append(None, [self.pixbuf_main, ii.tag, ii.text])
            for first in ii:
                row_first = self.treestore1.append(row_main, [self.pixbuf_first, first.tag, first.text])
                for second in first:
                    row_second = self.treestore1.append(row_first, [self.pixbuf_second, second.tag, second.text])
                    for third in second:
                        row_third = self.treestore1.append(row_second, [self.pixbuf_second, third.tag, third.text])
                        for fourth in third:
                            row_fourth = self.treestore1.append(row_third, [self.pixbuf_second, fourth.tag, fourth.text])                                            
                            for fifth in fourth:
                                row_fifth = self.treestore1.append(row_fourth, [self.pixbuf_second, fifth.tag, fifth.text])
                                
    # -----------------------------------------------------------------------------------------------------------------
    def createCylinder(self):

        GEN = generalClass(self)

        def closethis(widget):
            window.destroy()

        def applythis(widget):

            radius = radiusEntry.get_text()
            height = heightEntry.get_text()
            axis = axisCombo.child.get_text()
            xcenter = xEntry.get_text()
            ycenter = yEntry.get_text()
            zcenter = zEntry.get_text()
            name = nameEntry.get_text()
            if os.path.isfile(self.caseDir + '/' + name):
                generalClass(self).makeDialog('Warning!!! ' + name + ' is already exist. Use other name')
                return
            if name[-4:].upper() != '.STL':
                generalClass(self).makeDialog('Warning!!! ' + name + ' is not .stl')
                return
                                
            sourcename = self.installpath + '/common/unitcylinder.stl'
            targetname = self.caseDir + '/' + name

            xscale = float(radius) * 2
            cmd = "surfaceTransformPoints -scale '(" + str(xscale) + " " + str(xscale) + " " + height + ")' " + sourcename + " scale.stl \n"

            if axis == 'x':
                cmd = cmd + "surfaceTransformPoints -yawPitchRoll '(0 90 0)' scale.stl rot.stl \n"
            elif axis == 'y':
                cmd = cmd + "surfaceTransformPoints -yawPitchRoll '(0 0 90)' scale.stl rot.stl \n"
            else:
                cmd = cmd + "cp scale.stl rot.stl \n"

            cmd = cmd + "surfaceTransformPoints -translate '(" + xcenter + " " + ycenter + " " + zcenter + ")' rot.stl " + targetname
            generalClass(self).submitFile(cmd)

            window.destroy()
            self.addBoxCyliner(name)

        window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        window.set_title('Create cylinder STL file')
        window.set_border_width(10)
        window.set_position(gtk.WIN_POS_CENTER)
        window.set_transient_for(self.window)
        window.connect("delete_event", self.delete_event)

        mainvbox = gtk.VBox(False, 5)
        window.add(mainvbox)

        label = gtk.Label('Radius')
        radiusEntry = gtk.Entry()
        radiusEntry.set_text('0.5')
        radiusEntry.set_size_request(100, 28)
        hbox = gtk.HBox(False, 5)
        mainvbox.pack_start(hbox, False, False, 5)
        hbox.pack_start(label, False, False, 5)
        hbox.pack_end(radiusEntry, False, False, 5)

        label = gtk.Label('Height')
        heightEntry = gtk.Entry()
        heightEntry.set_text('1')
        hbox = gtk.HBox(False, 5)
        mainvbox.pack_start(hbox, False, False, 5)
        heightEntry.set_size_request(100, 28)
        hbox.pack_start(label, False, False, 5)
        hbox.pack_end(heightEntry, False, False, 5)

        label = gtk.Label('Axis')
        axisCombo = gtk.combo_box_entry_new_text()
        axisCombo.set_size_request(100, 28)
        axisCombo.append_text('x')
        axisCombo.append_text('y')
        axisCombo.append_text('z')
        axisCombo.set_active(2)
        hbox = gtk.HBox(False, 5)
        mainvbox.pack_start(hbox, False, False, 5)
        hbox.pack_start(label, False, False, 5)
        hbox.pack_end(axisCombo, False, False, 5)

        label = gtk.Label('Center      ')
        xEntry = gtk.Entry()
        yEntry = gtk.Entry()
        zEntry = gtk.Entry()
        xEntry.set_text('0')
        yEntry.set_text('0')
        zEntry.set_text('0')
        xEntry.set_size_request(80, 28)
        yEntry.set_size_request(80, 28)
        zEntry.set_size_request(80, 28)
        hbox = gtk.HBox(False, 5)
        mainvbox.pack_start(hbox, False, False, 5)
        hbox.pack_start(label, False, False, 5)
        hbox.pack_end(zEntry, False, False, 5)
        hbox.pack_end(yEntry, False, False, 0)
        hbox.pack_end(xEntry, False, False, 5)

        label = gtk.Label('fileName')
        nameEntry = gtk.Entry()
        nameEntry.set_text('cylinder.stl')
        hbox = gtk.HBox(False, 5)
        mainvbox.pack_start(hbox, False, False, 5)
        hbox.pack_start(label, False, False, 5)
        hbox.pack_end(nameEntry, True, True, 5)

        separator = gtk.HSeparator()
        separator.set_size_request(285, 5)
        mainvbox.pack_start(separator, False, True, 2)

        buttonbox = gtk.HBox(False, 5)
        mainvbox.pack_start(buttonbox, False, False, 5)

        app = gtk.Button(label='Apply')
        app.set_size_request(100, 30)
        cll = gtk.Button(label='Close')
        cll.set_size_request(100, 30)
        app.connect('clicked', applythis)
        cll.connect('clicked', closethis)
        buttonbox.pack_end(cll, False, False, 5)
        buttonbox.pack_end(app, False, False, 5)

        window.show_all()
    # -----------------------------------------------------------------------------------------------------------------
    def createBox(self):

        def closethis(widget):
            window.destroy()

        def applythis(widget):

            xlength = xlengthEntry.get_text()
            ylength = ylengthEntry.get_text()
            zlength = zlengthEntry.get_text()
            xcenter = xEntry.get_text()
            ycenter = yEntry.get_text()
            zcenter = zEntry.get_text()
            name = nameEntry.get_text()
            if os.path.isfile(self.caseDir + '/' + name):
                generalClass(self).makeDialog('Warning!!! ' + name + ' is already exist. Use other name')
                return
            if name[-4:].upper() != '.STL':
                generalClass(self).makeDialog('Warning!!! ' + name + ' is not .stl')
                return
                
            sourcename = self.installpath + '/common/unitCenterBox.stl'
            targetname = self.caseDir + '/' + name

            cmd = "surfaceTransformPoints -scale '(" + xlength + " " + ylength + " " + zlength + ")' " + sourcename + " scale.stl \n"
            cmd = cmd + "surfaceTransformPoints -translate '(" + xcenter + " " + ycenter + " " + zcenter + ")' scale.stl " + targetname
            generalClass(self).submitFile(cmd)

            window.destroy()
            self.addBoxCyliner(name)

        window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        window.set_title('Create cylinder STL file')
        window.set_border_width(10)
        window.set_position(gtk.WIN_POS_CENTER)
        window.set_transient_for(self.window)
        window.connect("delete_event", self.delete_event)

        mainvbox = gtk.VBox(False, 5)
        window.add(mainvbox)

        label = gtk.Label('x length')
        xlengthEntry = gtk.Entry()
        xlengthEntry.set_text('1')
        xlengthEntry.set_size_request(100, 28)
        hbox = gtk.HBox(False, 5)
        mainvbox.pack_start(hbox, False, False, 5)
        hbox.pack_start(label, False, False, 5)
        hbox.pack_end(xlengthEntry, False, False, 5)

        label = gtk.Label('y length')
        ylengthEntry = gtk.Entry()
        ylengthEntry.set_text('1')
        ylengthEntry.set_size_request(100, 28)
        hbox = gtk.HBox(False, 5)
        mainvbox.pack_start(hbox, False, False, 5)
        hbox.pack_start(label, False, False, 5)
        hbox.pack_end(ylengthEntry, False, False, 5)

        label = gtk.Label('z length')
        zlengthEntry = gtk.Entry()
        zlengthEntry.set_text('1')
        zlengthEntry.set_size_request(100, 28)
        hbox = gtk.HBox(False, 5)
        mainvbox.pack_start(hbox, False, False, 5)
        hbox.pack_start(label, False, False, 5)
        hbox.pack_end(zlengthEntry, False, False, 5)
        
        label = gtk.Label('Center      ')
        xEntry = gtk.Entry()
        yEntry = gtk.Entry()
        zEntry = gtk.Entry()
        xEntry.set_text('0')
        yEntry.set_text('0')
        zEntry.set_text('0')
        xEntry.set_size_request(80, 28)
        yEntry.set_size_request(80, 28)
        zEntry.set_size_request(80, 28)
        hbox = gtk.HBox(False, 5)
        mainvbox.pack_start(hbox, False, False, 5)
        hbox.pack_start(label, False, False, 5)
        hbox.pack_end(zEntry, False, False, 5)
        hbox.pack_end(yEntry, False, False, 0)
        hbox.pack_end(xEntry, False, False, 5)
        
        label = gtk.Label('fileName')
        nameEntry = gtk.Entry()
        nameEntry.set_text('box.stl')
        hbox = gtk.HBox(False, 5)
        mainvbox.pack_start(hbox, False, False, 5)
        hbox.pack_start(label, False, False, 5)
        hbox.pack_end(nameEntry, True, True, 5)

        separator = gtk.HSeparator()
        separator.set_size_request(285, 5)
        mainvbox.pack_start(separator, False, True, 2)

        buttonbox = gtk.HBox(False, 5)
        mainvbox.pack_start(buttonbox, False, False, 5)

        app = gtk.Button(label='Apply')
        app.set_size_request(100, 30)
        cll = gtk.Button(label='Close')
        cll.set_size_request(100, 30)
        app.connect('clicked', applythis)
        cll.connect('clicked', closethis)
        buttonbox.pack_end(cll, False, False, 5)
        buttonbox.pack_end(app, False, False, 5)

        window.show_all()                                
    #------------------------------------------------------------
    def addBoxCyliner(self, name):
    
        if not os.path.isdir(self.caseDir + '/constant/triSurface'):
            os.system('mkdir ' + self.caseDir + '/constant/triSurface')
            
        os.system('cp ' + self.caseDir + '/' + name + ' ' + self.caseDir + '/constant/triSurface/') 
        self.stlnames.append(name[:-4])
      
        self.createCadXML()
        self.makeTreeStore()        
        self.notebook.set_current_page(0)           
     
        selectedstl = []
        for i in range(len(self.stlnames)):
            selectedstl.append(self.caseDir + '/constant/triSurface/' + self.stlnames[i] + '.stl')       
                
        self.renderer.RemoveAllViewProps()
        self.Actors = self.VTK_All.showSTLMultiForSnappy(selectedstl)
        for ii in self.Actors:
            self.renderer.AddActor(ii)
        self.gvtk.Initialize()
        self.resetSize()
        
        generalClass(self).indentXML(self.root) 
        self.tree.write(self.caseDir + '/setup_snappy.xml')                                                 
    #-----------------------------------------------------------------------------------------------------
    def readSetting(self):

        xmlName = 'setup_snappy.xml'
        
        dialog = gtk.FileChooserDialog("Select setup file", None, gtk.FILE_CHOOSER_ACTION_OPEN,
                 (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        filters = gtk.FileFilter()
        filters.set_name('setup file')
        filters.add_pattern(xmlName)
        dialog.add_filter(filters) 
        
        dialog.set_default_response(gtk.RESPONSE_OK)
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            filename = dialog.get_filename()
        elif response == gtk.RESPONSE_CANCEL:
            filename = None
        dialog.destroy()        

        if filename:
            os.system('cp ' + filename + ' ' + self.caseDir + '/' + xmlName) 

            self.setfolder()
            self.loadXML()            
            
            self.stlnames = []
            for ii in self.stl:
                self.stlnames.append(ii.tag)
                
            self.boxActorDict = {}
            for ii in self.box:
                boxActor = self.VTK_All.showBoxNew(ii.find('minx').text, ii.find('miny').text, ii.find('minz').text,
                    ii.find('maxx').text, ii.find('maxy').text, ii.find('maxz').text)
                self.boxActorDict[ii.tag] = boxActor                
                    
            for ii in self.freevbox.get_children():
                self.freevbox.remove(ii)
            self.cadPart()
            self.freevbox.add(self.cadbox)

            self.window.show_all()
            self.notebook.set_current_page(0) 
            
            self.resetRenderer() 


   
