#-*-coding:utf8-*-

from common.fromAll_snappy  import *

class writeFileClassSnappy:

    def __init__(self,caseDir):
        self.caseDir = caseDir
	
    #-------------------------------------------------------------------------------------
    def foamComment(self):
        fc=[]
        fc.append('/*--------------------------------*- C++ -*----------------------------------*\ \n')
        fc.append('| =========                 |                                                 |\n')
        fc.append('| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |\n')
        fc.append('|  \\    /   O peration     | Version:  2.3.0                                 |\n')
        fc.append('|   \\  /    A nd           | Web:      www.OpenFOAM.org                      |\n')
        fc.append('|    \\/     M anipulation  |                                                 |\n')
        fc.append('\*---------------------------------------------------------------------------*/\n')	
        return fc
    #------------------------------------------------------------------------------
    def header(self, location, obj):
        he=[]
        he.append('FoamFile\n')
        he.append('{\n')
        he.append('    version     2.0;\n')
        he.append('    format      ascii;\n')
        he.append('    class       dictionary;\n')
        he.append('    location    "' + location + '";\n')
        he.append('    object      ' + obj + ';\n')
        he.append('}\n')
        he.append('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n')
        return he
    #------------------------------------------------------------------------------
    def BCheader(self, classtype, obj):
        he=[]
        he.append('FoamFile\n')
        he.append('{\n')
        he.append('    version     2.0;\n')
        he.append('    format      ascii;\n')
        he.append('    class       ' + classtype + ';\n')
        he.append('    object      ' + obj + ';\n')
        he.append('}\n')
        he.append('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n')
        return he
    #------------------------------------------------------------------------------
    def decomposeParDict(self, ncore):
        fcom = self.foamComment()
        head = self.header('system', 'decomposeParDict')
        hhh = fcom + head
        f = open(self.caseDir + '/system/decomposeParDict', 'w')
        for aaa in hhh:
            f.write(aaa)
        f.write('numberOfSubdomains     ' + ncore + ';\n')
        f.write('method                 scotch;\n')
        f.close()
    #------------------------------------------------------------------------------
    def controlDictFile(self, run):

        fcom = self.foamComment()
        head = self.header('system', 'controlDict')
        hhh = fcom + head
        f = open(self.caseDir + '/system/controlDict', 'w')
        for aaa in hhh:
            f.write(aaa)
        f.write('application        simpleFoam;\n')
        f.write('startFrom          startTime;\n')
        f.write('startTime          ' + run.find('startFrom').text + ';\n')
        f.write('stopAt             endTime;\n')
        f.write('endTime            100;\n')
        f.write('deltaT             1;\n')
        f.write('writeControl       timeStep;\n')
        f.write('writeInterval      100;\n')
        f.write('purgeWrite         0;\n')
        f.write('writeFormat        ' + run.find('writeFormat').text + ';\n')
        f.write('writePrecision     ' + run.find('writePrecision').text + ';\n')  
        f.write('writeCompression   ' + run.find('dataCompression').text + ';\n')
        f.write('timeFormat         general;\n')
        f.write('timePrecision      6;\n')
        f.write('runTimeModifiable  yes;\n')
        f.close()
    #-------------------------------------------------------------------------------------------------
    def makeFeatureFile(self, stlnames, featureAngle, nonManifoldEdges, openEdges):
    
        f = open(self.caseDir + '/system/surfaceFeatureExtractDict', 'w')
        f.write('/*--------------------------------*- C++ -*----------------------------------*\ \n')
        f.write('| =========                 |                                                 | \n')
        f.write('| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           | \n')
        f.write('|  \\    /   O peration     | Version:  2.3.x                                 | \n')
        f.write('|   \\  /    A nd           | Web:      www.OpenFOAM.org                      | \n')
        f.write('|    \\/     M anipulation  |                                                 | \n')
        f.write('\*---------------------------------------------------------------------------*/ \n')
        f.write('FoamFile \n')
        f.write('{ \n')
        f.write('    version     2.0; \n')
        f.write('    format      ascii; \n')
        f.write('    class       dictionary; \n')
        f.write('    object      surfaceFeatureExtractDict; \n')
        f.write('} \n')
        f.write('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * // \n')

        for i in range(len(stlnames)):
            f.write(stlnames[i] + '.stl\n')
            f.write('{ \n')
            f.write('    extractionMethod    extractFromSurface;\n')
            f.write('    extractFromSurfaceCoeffs \n')
            f.write('    {\n')
            f.write('        includedAngle   ' + featureAngle + ';\n')
            f.write('    }\n')
            f.write('    subsetFeatures\n')
            f.write('    {\n')
            f.write('        nonManifoldEdges    ' + nonManifoldEdges + ';\n')
            f.write('        openEdges           ' + openEdges + ';\n')
            f.write('    }\n')
            f.write('    writeObj                yes;\n')
            f.write('}\n')

        f.close()                     
    #-------------------------------------------------------------------------------------------------
    def snappyHexMeshDict(self, root):
        
        cad = root.find('cad')
        stl = cad.find('stlregions')
        box = cad.find('boxes')
        castel = root.find('castellate')
        surfaceLevel = castel.find('surfaceLevel')
        featureLevel = castel.find('featureLevel')
        boxLevel = castel.find('boxLevel')
        snap = root.find('snap')
        layer = root.find('layer')
        layerPatch = layer.find('patches')
        adv = root.find('advanced')
        run = root.find('runConditions')
        
        stlnames = []
        for ii in surfaceLevel:
            stlnames.append(ii.tag)


        f = open(self.caseDir + '/system/snappyHexMeshDict', 'w')
        f.write('/*--------------------------------*- C++ -*----------------------------------*\ \n')
        f.write('| =========                 |                                                 | \n')
        f.write('| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           | \n')
        f.write('|  \\    /   O peration     | Version:  2.3.x                                 | \n')
        f.write('|   \\  /    A nd           | Web:      www.OpenFOAM.org                      | \n')
        f.write('|    \\/     M anipulation  |                                                 | \n')
        f.write('\*---------------------------------------------------------------------------*/ \n')
        f.write('FoamFile \n')
        f.write('{ \n')
        f.write('    version     2.0; \n')
        f.write('    format      ascii; \n')
        f.write('    class       dictionary; \n')
        f.write('    object      snappyHexMeshDict; \n')
        f.write('} \n')
        f.write('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * // \n')

        f.write('castellatedMesh ' + run.find('castellate').text + ';\n')
        f.write('snap            ' + run.find('snap').text + ';\n')
        f.write('addLayers       ' + run.find('addLayer').text + ';\n')

        f.write('geometry \n')
        f.write('{ \n')
                
        for i in range(len(stlnames)):
            f.write('    '+stlnames[i] + '.stl\n')
            f.write('    { \n')
            f.write('        type    triSurfaceMesh;\n')
            f.write('        name    ' + stlnames[i] + ';\n')                                               
            f.write('    } \n')

        if len(box) != 0:
            for i in range(len(box)):
                name = box[i].tag
                em = box.find(name)
                f.write('    ' + box[i].tag + '\n')
                f.write('    {\n')
                if em.find('type').text == 'box':
                    f.write('        type    searchableBox;\n')
                    f.write('        min     (' + em.find('minx').text + ' ' + em.find('miny').text + ' ' + em.find('minz').text + ');\n')
                    f.write('        max     (' + em.find('maxx').text + ' ' + em.find('maxy').text + ' ' + em.find('maxz').text + ');\n')	
                elif em.find('type').text == 'sphere':
                    f.write('        type    searchableSphere;\n')
                    f.write('        centre  (' + em.find('centerx').text + ' ' + em.find('centery').text + ' ' + em.find('centerz').text + ');\n')
                    f.write('        radius  ' + em.find('radius').text + ';\n')	
                elif em.find('type').text == 'cylinder':
                    cx = float(em.find('centerx').text)
                    cy = float(em.find('centery').text)
                    cz = float(em.find('centerz').text)
                    h = float(em.find('height').text)
                    axis = em.find('axis').text
                    
                    if axis == 'x':
                        point1x = cx - 0.5 * h
                        point1y = cy
                        point1z = cz
                        point2x = cx + 0.5 * h
                        point2y = cy
                        point2z = cz
                    elif axis == 'y':
                        point1y = cy - 0.5 * h
                        point1x = cx
                        point1z = cz
                        point2y = cy + 0.5 * h
                        point2x = cx
                        point2z = cz
                    elif axis == 'z':
                        point1z = cz - 0.5 * h
                        point1y = cy
                        point1x = cx
                        point2z = cz + 0.5 * h
                        point2y = cy
                        point2x = cx                                  
                    
                    f.write('        type    searchableCylinder;\n')
                    f.write('        point1  (' + str(point1x) + ' ' + str(point1y) + ' ' + str(point1z) + ');\n')
                    f.write('        point2  (' + str(point2x) + ' ' + str(point2y) + ' ' + str(point2z) + ');\n')
                    f.write('        radius  ' + em.find('radius').text + ';\n')
                                        
                f.write('    }\n')
        f.write('}; \n')
        #---------------------------------------
        f.write('castellatedMeshControls \n')
        f.write('{ \n')
        f.write('    maxLocalCells ' + castel.find('maxLocalCells').text + ';\n')
        f.write('    maxGlobalCells ' + castel.find('maxGlobalCells').text + ';\n')
        f.write('    minRefinementCells ' + castel.find('minRefinementCells').text + ';\n')
        f.write('    nCellsBetweenLevels ' + castel.find('nCellsBetweenLevels').text + ';\n')

        f.write('    features\n')
        f.write('    (\n')
        for i in range(len(featureLevel)):
            f.write('        {\n')
            f.write('            file "' + featureLevel[i].tag + '.eMesh";\n')
            f.write('            levels ((0.01 ' + featureLevel[i].text + '));\n')
            f.write('        } \n')
        f.write('    );\n')

        f.write('    refinementSurfaces \n')
        f.write('    { \n')
        for i in range(len(surfaceLevel)):
            f.write('        ' + surfaceLevel[i].tag + '\n')
            f.write('        {\n')
            f.write('            level (' + surfaceLevel[i].text + ' ' + surfaceLevel[i].text + ');\n')
            nameEm = stl.find(surfaceLevel[i].tag)
            patchtype = nameEm.find('type').text
            if patchtype == 'wall':
                f.write('            patchInfo\n')
                f.write('            {\n')
                f.write('                type    wall;\n')
                f.write('            }\n')
            elif patchtype == 'patch':
                f.write('            patchInfo\n')
                f.write('            {\n')
                f.write('                type    patch;\n')
                f.write('            }\n')
            elif patchtype == 'baffle':
                f.write('            faceZone    ' + surfaceLevel[i].tag + ';\n')
                f.write('            faceType    baffle;\n')
            elif patchtype == 'internal':
                f.write('            faceZone    ' + surfaceLevel[i].tag + ';\n')
                f.write('            faceType    internal;\n')
            elif patchtype == 'cellZone':
                f.write('            cellZone    ' + surfaceLevel[i].tag + ';\n')
                f.write('            faceZone    ' + surfaceLevel[i].tag + ';\n')                
                em = stl.find(surfaceLevel[i].tag)                
                f.write('            cellZoneInside    inside;\n')
            f.write('        } \n')
        f.write('    }\n')

        f.write('    resolveFeatureAngle ' + castel.find('resolveFeatureAngle').text + ';\n')

        f.write('    refinementRegions\n')
        f.write('    {\n')
        for i in range(len(boxLevel)):
            f.write('        ' + boxLevel[i].tag + '\n')
            f.write('        {\n')
            f.write('            mode    inside;\n')
            f.write('            levels  ((1E15 ' + boxLevel[i].text + '));\n')
            f.write('        }\n')
        for i in range(len(stl)):
            if stl[i].find('type').text == 'cellZone':
                f.write('        ' + stl[i].tag + '\n')
                f.write('        {\n')
                f.write('            mode    ' + stl[i].find('mode').text + ';\n')
                f.write('            levels  ((' + stl[i].find('distance').text + ' ' + stl[i].find('level').text + '));\n')
                f.write('        }\n')
        f.write('    }\n')
                    
        f.write('    locationInMesh (' + castel.find('locationx').text + ' ' + castel.find('locationy').text + ' ' + castel.find('locationz').text + ');\n')
        f.write('    allowFreeStandingZoneFaces ' + castel.find('allowFreeStanding').text + ';\n')
        f.write('}\n')
        f.write('\n')
        #---------------------------------------
        f.write('snapControls\n')
        f.write('{\n')
        f.write('    nSmoothPatch           ' + snap.find('nSmoothPatch').text + ';\n')
        f.write('    tolerance              ' + snap.find('tolerance').text + ';\n')
        f.write('    nSolveIter             ' + snap.find('nSolveIter').text + ';\n')
        f.write('    nRelaxIter             ' + snap.find('nRelaxIter').text + ';\n')
        f.write('    nFeatureSnapIter       ' + snap.find('nFeatureSnapIter').text + ';\n')
        f.write('    implicitFeatureSnap    ' + snap.find('implicitFeatureSnap').text + ';\n')
        f.write('    explicitFeatureSnap    ' + snap.find('explicitFeatureSnap').text + ';\n')
        f.write('    multiRegionFeatureSnap ' + snap.find('multiRegionFeatureSnap').text + ';\n')
        f.write('}\n')
        f.write('\n')
        #---------------------------------------
        f.write('addLayersControls\n')
        f.write('{\n')
        f.write('    relativeSizes ' + layer.find('relativeSize').text + ';\n')
        f.write('    expansionRatio            ' + layer.find('expansionRatio').text + ';\n')
        
        if layer.find('thicknessMethod').text == 'firstLayer':
            f.write('    firstLayerThickness       ' + layer.find('firstThickness').text + ';\n')
        elif layer.find('thicknessMethod').text == 'finalLayer':
            f.write('    finalLayerThickness       ' + layer.find('finalThickness').text + ';\n')
        if layer.find('thicknessMethod').text == 'overall':
            f.write('    thickness       ' + layer.find('overallThickness').text + ';\n')

        f.write('    minThickness              ' + layer.find('minThickness').text + ';\n')

        f.write('    layers\n')
        f.write('    {\n')

        for i in range(len(layerPatch)):
            if layerPatch[i].text == 'yes':
                f.write('        ' + layerPatch[i].tag + '\n')
                f.write('        { \n')
                f.write('            nSurfaceLayers ' + layer.find('nlayer').text + ';\n')
                f.write('        } \n')
        f.write('    }\n')

        f.write('    nGrow                     ' + layer.find('nGrow').text + ';\n')
        f.write('    featureAngle              ' + layer.find('featureAngle').text + ';\n')
        f.write('    nRelaxIter                ' + layer.find('nRelaxIter').text + ';\n')
        f.write('    nSmoothSurfaceNormals     ' + layer.find('nSmoothSurfaceNormals').text + ';\n')
        f.write('    nSmoothNormals            ' + layer.find('nSmoothNormals').text + ';\n')
        f.write('    nSmoothThickness          ' + layer.find('nSmoothThickness').text + ';\n')
        f.write('    maxFaceThicknessRatio     ' + layer.find('maxFaceThicknessRatio').text + ';\n')
        f.write('    maxThicknessToMedialRatio ' + layer.find('maxThicknessToMedialRatio').text + ';\n')
        f.write('    minMedianAxisAngle        ' + layer.find('minMedianAxisAngle').text + ';\n')
        f.write('    nBufferCellsNoExtrude     ' + layer.find('nBufferCellsNoExtrude').text + ';\n')
        f.write('    nLayerIter                ' + layer.find('nLayerIter').text + ';\n')
        f.write('    nRelaxedIter              ' + layer.find('nRelaxedIter').text + ';\n')

        f.write('}\n')
        f.write('\n')
        #---------------------------------------
        f.write('meshQualityControls\n')
        f.write('{\n')
        f.write('    maxNonOrtho           ' + adv.find('maxNonOrtho').text + ';\n')
        f.write('    maxBoundarySkewness   ' + adv.find('maxBoundarySkewness').text + ';\n')
        f.write('    maxInternalSkewness   ' + adv.find('maxInternalSkewness').text + ';\n')
        f.write('    maxConcave            ' + adv.find('maxConcave').text + ';\n')
        f.write('    minVol                ' + adv.find('minVol').text + ';\n')
        f.write('    minTetQuality         ' + adv.find('minTetQuality').text + ';\n')
        f.write('    minArea               ' + adv.find('minArea').text + ';\n')
        f.write('    minTwist              ' + adv.find('minTwist').text + ';\n')
        f.write('    minDeterminant        ' + adv.find('minDeterminant').text + ';\n')
        f.write('    minFaceWeight         ' + adv.find('minFaceWeight').text + ';\n')
        f.write('    minVolRatio           ' + adv.find('minVolRatio').text + ';\n')
        f.write('    minTriangleTwist      ' + adv.find('minTriangleTwist').text + ';\n')
        f.write('\n')

        f.write('    nSmoothScale   ' + adv.find('nSmoothScale').text + ';\n')
        f.write('    errorReduction ' + adv.find('errorReduction').text + ';\n')
        f.write('    relaxed\n')
        f.write('    {\n')
        f.write('        maxNonOrtho  ' + adv.find('relaxedMaxNonOrtho').text + ';\n')
        f.write('    }\n')
        f.write('}\n')
        f.write('\n')

        f.write('mergeTolerance ' + adv.find('mergeTolerance').text + ';\n')

        f.close()
    #-------------------------------------------------------------------------------------------------
    def createPatchNone(self):
    
        f = open(self.caseDir + '/system/createPatchDict', 'w')
        f.write('/*--------------------------------*- C++ -*----------------------------------*\ \n')
        f.write('| =========                 |                                                 | \n')
        f.write('| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           | \n')
        f.write('|  \\    /   O peration     | Version:  2.3.x                                 | \n')
        f.write('|   \\  /    A nd           | Web:      www.OpenFOAM.org                      | \n')
        f.write('|    \\/     M anipulation  |                                                 | \n')
        f.write('\*---------------------------------------------------------------------------*/ \n')
        f.write('FoamFile \n')
        f.write('{ \n')
        f.write('    version     2.0; \n')
        f.write('    format      ascii; \n')
        f.write('    class       dictionary; \n')
        f.write('    object      createPatchDict; \n')
        f.write('} \n')
        f.write('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * // \n')
        f.write('pointSync  false;\n')
        f.write('patches\n')
        f.write('( );\n')
        f.close()
    #-------------------------------------------------------------------------------------------------
    def createPatch(self, oldname, newname, patchtype):
    
        f = open(self.caseDir + '/system/createPatchDict', 'w')
        f.write('/*--------------------------------*- C++ -*----------------------------------*\ \n')
        f.write('| =========                 |                                                 | \n')
        f.write('| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           | \n')
        f.write('|  \\    /   O peration     | Version:  2.3.x                                 | \n')
        f.write('|   \\  /    A nd           | Web:      www.OpenFOAM.org                      | \n')
        f.write('|    \\/     M anipulation  |                                                 | \n')
        f.write('\*---------------------------------------------------------------------------*/ \n')
        f.write('FoamFile \n')
        f.write('{ \n')
        f.write('    version     2.0; \n')
        f.write('    format      ascii; \n')
        f.write('    class       dictionary; \n')
        f.write('    object      createPatchDict; \n')
        f.write('} \n')
        f.write('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * // \n')
        f.write('pointSync  false;\n')
        f.write('patches\n')
        f.write('(\n')
        
        f.write('    {\n')
        f.write('        name ' + newname + ';\n')
        f.write('        patchInfo\n')
        f.write('        {\n')
        f.write('             type    ' + patchtype + ';\n')
        f.write('        }\n')
        f.write('        constructFrom patches;\n')
        f.write('        patches (' + oldname + ');\n')
        f.write('    }\n')
        f.write(');\n')
        f.close()	        	
    #--------------------------------------------------------------------------------------  
    def basicfvsolution(self):
    
        fcom = self.foamComment()
        head = self.header('system', 'fvSolution')
        hhh = fcom + head
        f = open(self.caseDir + '/system/fvSolution', 'w')
        for aaa in hhh:
            f.write(aaa)
            
        f.write('solvers\n')
        f.write('{\n')
        f.write('    "rho.*"\n')
        f.write('    {\n')
        f.write('        solver          PCG;\n')
        f.write('        preconditioner  DIC;\n')
        f.write('        tolerance       1e-7;\n')
        f.write('        relTol          0;\n')
        f.write('    }\n')
        f.write('    "(p|p_rgh)"\n')
        f.write('    {\n')
       	f.write('        solver                GAMG;\n')
        f.write('        tolerance             1e-7;\n')
        f.write('        relTol                0.1;\n')
        f.write('        minIter               0;\n')
        f.write('        maxIter               50;\n')
        f.write('        smoother              symGaussSeidel;\n')
        f.write('        nPreSweeps            0;\n')
        f.write('        nPostSweeps           2;\n')
        f.write('        cacheAgglomeration    true;\n')
        f.write('        agglomerator          faceAreaPair;\n')
        f.write('        nCellsInCoarsestLevel 10;\n')
        f.write('        mergeLevels           1;\n')
        f.write('    };\n')
        f.write('    "(pFinal|p_rghFinal)"\n')
        f.write('    {\n')
       	f.write('        $p_rgh;\n')
        f.write('        relTol                0;\n')
        f.write('    };\n')
        f.write('    "(U|k|epsilon|omega)"\n')
        f.write('    {\n')
        f.write('        solver          smoothSolver;\n')
        f.write('        smoother        symGaussSeidel;\n')
        f.write('        tolerance       1e-7;\n')
        f.write('        relTol          0.1;\n')
        f.write('    };\n')
        f.write('    "(U|k|epsilon|omega)Final"\n')
        f.write('    {\n')
      	f.write('        $U;\n')
        f.write('        relTol                0;\n')
        f.write('    };\n')
        f.write('    h\n')
        f.write('    {\n')
        f.write('        solver          smoothSolver;\n')
        f.write('        smoother        symGaussSeidel;\n')
        f.write('        tolerance       1e-7;\n')
        f.write('        relTol          0.01;\n')
        f.write('    };\n')
        f.write('    hFinal\n')
        f.write('    {\n')
       	f.write('        $U;\n')
        f.write('        relTol                0;\n')
        f.write('    };\n')
        f.write('}\n')
        f.write('SIMPLE\n')
        f.write('{\n')
        f.write('    nNonOrthogonalCorrectors 0;\n')
        f.write('    pRefCell	0;\n')
        f.write('    pRefValue	0;\n')  
        f.write('    residualControl\n')
        f.write('    {\n')
        f.write('        p      1e-3;\n')
        f.write('        p_rgh  1e-3;\n')        
        f.write('        U      1e-3;\n')
        f.write('        h      1e-3;\n')        
        f.write('        "(k|epsilon|omega)"    1e-3;\n')
        f.write('    }\n')                
        f.write('}\n')
        f.write('PIMPLE\n')
        f.write('{\n')
        f.write('    nNonOrthogonalCorrectors 0;\n')
        f.write('    nOuterCorrectors 1;\n')
        f.write('    nCorrectors 2;\n')
        f.write('    pRefCell	0;\n')
        f.write('    pRefValue	0;\n')        
        f.write('}\n')
        f.write('relaxationFactors\n')
        f.write('{\n')
        f.write('    fields\n')
        f.write('    {\n')
        f.write('        "rho.*"      1.0;\n')
        f.write('        "p.*"        0.3;\n')        
        f.write('    }\n')
        f.write('    equations\n')
        f.write('    {\n')
        f.write('        "(U|k|epsilon|omega).*"    0.7;\n')
        f.write('        "h.*"         1.0;\n')
        f.write('    }\n')
        f.write('}\n')
        f.close()
    #--------------------------------------------------------------------------------------  
    def basicfvschemes(self):
        
        fcom = self.foamComment()
        head = self.header('system', 'fvSchemes')
        hhh = fcom + head
        f = open(self.caseDir + '/system/fvSchemes', 'w')
        for aaa in hhh:
            f.write(aaa)	
            
        f.write('ddtSchemes\n')
        f.write('{\n')
        f.write('    default         steadyState;\n')
        f.write('}\n')
        f.write('gradSchemes\n')
        f.write('{\n')
        f.write('    default         Gauss linear;\n')
        f.write('    limitedGrad     cellLimited Gauss linear 1;\n')
        f.write('}\n')
        f.write('divSchemes\n')
        f.write('{\n')
        f.write('    default	      Gauss linear;\n')
        f.write('    div(phi,U)       bounded Gauss upwind;\n')
        f.write('    div(phi,k)       bounded Gauss upwind;\n')
        f.write('    div(phi,K)       bounded Gauss upwind;\n')
        f.write('    div(phi,epsilon) bounded Gauss upwind;\n')
        f.write('    div(phi,omega)   bounded Gauss upwind;\n')
        f.write('    div(phi,h)       bounded Gauss upwind;\n')
        f.write('}\n')
        f.write('laplacianSchemes\n')
        f.write('{\n')
        f.write('    default         Gauss linear corrected;\n')
        f.write('}\n')
        f.write('interpolationSchemes\n')
        f.write('{\n')
        f.write('    default         linear;\n')
        f.write('}\n')
        f.write('snGradSchemes\n')
        f.write('{\n')
        f.write('    default         corrected;\n')
        f.write('}\n')
        f.write('fluxRequired\n')
        f.write('{\n')
        f.write('    default         no;\n')
        f.write('    p;\n')
        f.write('    p_rgh;\n')
        f.write('    pcorr;\n')
        f.write('}\n')
        f.close()
    #--------------------------------------------------------------------------------------  
    def basiccontroldict(self):
    
        fcom = self.foamComment()
        head = self.header('system', 'controlDict')
        hhh = fcom + head
        f = open(self.caseDir + '/system/controlDict', 'w')
        for aaa in hhh:
            f.write(aaa)	
            
        f.write('application     simpleFoam;\n')
        f.write('startFrom       latestTime;\n')
        f.write('startTime       0;\n')
        f.write('stopAt          endTime;\n')
        f.write('endTime         10000;\n')
        f.write('deltaT          1;\n')
        f.write('writeControl    runTime;\n')
        f.write('writeInterval   1000;\n')
        f.write('purgeWrite      10;\n')
        f.write('writeFormat     ascii;//binary;\n')
        f.write('writePrecision  6;\n')
        f.write('writeCompression uncompressed;\n')
        f.write('timeFormat      general;\n')
        f.write('timePrecision   6;\n')
        f.write('runTimeModifiable yes;\n')
        f.write('adjustTimeStep no;\n')
        f.write('maxCo 1.0;\n')
        f.write('maxDeltaT 1.0;\n')
        f.close()
    #--------------------------------------------------------------------------------------  
    def topoSetDict(self, dic):
        fcom = self.foamComment()
        head = self.header('system', 'topoSetDict')
        hhh = fcom + head
        f = open(self.caseDir + '/system/topoSetDict', 'w')
        for aaa in hhh:
            f.write(aaa)	
            
        f.write('actions\n')
        f.write('(\n')
        f.write('    {\n')
        f.write('        name       ' + dic['name'] + ';\n')
        f.write('        type       cellSet;\n')
        f.write('        action     new;\n')
        f.write('        source     boxToCell;\n')
        f.write('        sourceInfo\n')
        f.write('        {\n')
        f.write('             box (' + dic['min'][0] + ' ' + dic['min'][1] + ' ' + dic['min'][2] + ') (' + dic['max'][0] + ' ' + dic['max'][1] + ' ' + dic['max'][2] + ');\n')
        f.write('        }\n')
        f.write('    }\n')
        f.write(');\n')
        f.close()
    #--------------------------------------------------------------------------------------  
    def refineMeshDict(self, dic):
    
        fcom = self.foamComment()
        head = self.header('system', 'refineMeshDict')
        hhh = fcom + head
        f = open(self.caseDir + '/system/refineMeshDict', 'w')
        for aaa in hhh:
            f.write(aaa)	
            
        f.write('set    ' + dic['cellset'] + ';\n')
        f.write('coordinateSystem   global;\n')
        f.write('globalCoeffs\n')
        f.write('{\n')
        f.write('    tan1   (1 0 0);\n')
        f.write('    tan2   (0 1 0);\n')
        f.write('}\n')
        f.write('directions\n')
        f.write('(\n')
        f.write('    tan1\n')
        f.write('    tan2\n')
        f.write('    normal\n')
        f.write(');\n') 
        f.write('useHexTopology     ' + dic['useHexTopology'] + ';\n')
        f.write('geometricCut       ' + dic['geometricCut'] + ';\n')
        f.write('writeMesh          ' + dic['writeMesh'] + ';\n')                           
        f.close() 
        
               
